@@ -45,6 +45,8 @@ static char *dump_sync_map(void);
 void pool_init_session_context(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
 {
 	session_context = &session_context_d;
+	ProcessInfo *process_info;
+	int node_id;
 
 	/* Get Process context */
 	session_context->process_context = pool_get_process_context();
@@ -73,22 +75,28 @@ void pool_init_session_context(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *
 	/* Initialize sent message list */
 	init_sent_message_list();
 
+	process_info = pool_get_my_process_info();
+
+	if (!process_info)
+		ereport(ERROR,
+				(errmsg("failed to get process info for current process")));
+
 	/* Choose load balancing node if necessary */
 	if (pool_config->load_balance_mode)
 	{
-		ProcessInfo *process_info = pool_get_my_process_info();
-		if (!process_info)
-			ereport(ERROR,
-					(errmsg("failed to get process info for current process")));
-
-		session_context->load_balance_node_id = 
-			process_info->connection_info->load_balancing_node =
-			select_load_balancing_node();
-
-		ereport(DEBUG1,
-			(errmsg("initializing session context"),
-				 errdetail("selected load balancing node: %d", backend->info->load_balancing_node)));
+		node_id = select_load_balancing_node();
 	}
+	else
+	{
+		node_id = STREAM? PRIMARY_NODE_ID: MASTER_NODE_ID;
+	}
+
+	session_context->load_balance_node_id =
+		process_info->connection_info->load_balancing_node = node_id;
+
+	ereport(DEBUG1,
+			(errmsg("initializing session context"),
+			 errdetail("selected load balancing node: %d", backend->info->load_balancing_node)));
 
 	/* Unset query is in progress */
 	pool_unset_query_in_progress();
